
package ittp;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;

/**
 *  TCP_Client
 *  
 *  Simple client for testing HTTP-like protocols over TCP.  
 *
 *  @author Jesper Kampfeldt, modified by Hans Henrik Lvengreen
 *  @version 1.0, November 4, 2004
 */


public class TCP_Client extends JFrame {

    final static String version = "1.0";

    static final String DEF_HOST = "localhost";
    static final int DEF_PORT = 36000;

    JPanel addressPanel = null;
    JPanel mainPanel = null;
    JPanel buttonPanel = null;
    JPanel mainPane = null;
    JTextArea responseText = null;
    JTextArea cmdText = null;
    JTextField hostField;
    JTextField portField;
    Socket sck = null;
    JButton connect = null;
    JButton disconnect = null;
    JButton send = null;

    Sender sender = null;

    public TCP_Client() {

        mainPane = mainPane();
        setTitle("TCP Client" + " v." + version);
        setContentPane(mainPane);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); 
	                       // Exit program when frame is closed
        setLocation(200, 200); // located at (200, 200)

	disconnect.setEnabled(false);
	send.setEnabled(false);

        pack();                // Frame is ready. Pack it up for display.
        setVisible(true);      // Make it visible
   }

    private JPanel addressPanel() {
        JPanel ap = new JPanel(new FlowLayout());
        JLabel hostLabel = new JLabel("Host:");
        JLabel portLabel = new JLabel("Port:");
        hostField = new JTextField(20);
        portField = new JTextField(6);
        hostField.setText(DEF_HOST);
        portField.setText(""+DEF_PORT);
        ap.add(hostLabel);
        ap.add(hostField);
        ap.add(portLabel);
        ap.add(portField);
        return ap;

    }

    private JPanel mainPanel() {
        JPanel mp = new JPanel(new GridLayout(2, 1));
        JPanel resPanel = new JPanel(new BorderLayout());
        JPanel cmdPanel = new JPanel(new BorderLayout());
        resPanel.add(new JLabel("Response message:"));

        //Builds the command textarea
        cmdText = new JTextArea(10, 40);
        cmdText.setLineWrap(true);
        cmdText.setEditable(true);
        cmdText.setForeground(Color.blue);
        JScrollPane cmdTextPane = new JScrollPane(cmdText,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        //Builds the response textarea
        responseText = new JTextArea(20, 40);
        responseText.setLineWrap(true);
        responseText.setEditable(false);
        responseText.setForeground(Color.blue);
        JScrollPane responseTextPane = new JScrollPane(responseText,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        //Builds the cmdPanel
        cmdPanel.add(new JLabel("Command message"), BorderLayout.NORTH);
        cmdPanel.add(cmdTextPane, BorderLayout.CENTER);

        //Builds the resPanel
        resPanel.add(new JLabel("Response message"), BorderLayout.NORTH);
        resPanel.add(responseTextPane, BorderLayout.CENTER);

        //Builds the main panel
        mp.add(cmdPanel);
        mp.add(resPanel);
        mp.setPreferredSize(new Dimension(500, 300));
        return mp;
    }

    private JPanel buttonPanel() {
        JPanel bp = new JPanel(new FlowLayout());
        send = new JButton("Send");
        connect = new JButton("Connect");
        disconnect = new JButton("Disconnect");
        disconnect.setActionCommand("disconnect");
        send.setActionCommand("send");
        connect.setActionCommand("connect");
        bp.add(connect);
        bp.add(disconnect);
        bp.add(send);
        ActionAdapter actA = new ActionAdapter();
        send.addActionListener(actA);
        connect.addActionListener(actA);
        disconnect.addActionListener(actA);
        return bp;
    }

    private JPanel mainPane() {
        //initiate the panels
        addressPanel = addressPanel();
        mainPanel = mainPanel();
        buttonPanel = buttonPanel();

        JPanel mp = new JPanel(new BorderLayout());
        mp.add(addressPanel, BorderLayout.NORTH);
        mp.add(mainPanel, BorderLayout.CENTER);
        mp.add(buttonPanel, BorderLayout.SOUTH);
        return mp;
    }


    public static void main(String[] args) {
        new TCP_Client();
    }

    void error(String msg) {
	responseText.setForeground(Color.red);
	responseText.setText(msg + "\n");
	System.out.println(msg);
    }

    void OK(String msg) {
	responseText.setForeground(new Color(0,150,0));
	responseText.setText("OK "+ msg + "\n");
	System.out.println(msg);
    }

    void doConnect() {
	String host = hostField.getText();
	int port = DEF_PORT;
	try {
	    port = Integer.parseInt(portField.getText());
	} catch (Exception e) {
	    error("Port field not a proper number.  Using " + 
		  DEF_PORT + " instead");
	}
	
	try {
	    sck = new Socket(host, port);
	    OK("Connected to " + host + ":" + port);
	    connect.setEnabled(false);
	    disconnect.setEnabled(true);
	    send.setEnabled(true);
	} catch (Exception e) {
	    error("Connection to " + host + ":" + port + " failed: \n  " + e);
	}
    }

    void doDisconnect() {
	try {
	    if (sck!=null) sck.close();
	    OK("");
	} catch (IOException e) {
	    error("Could not close connection " + 4);
	    return;
	}
	connect.setEnabled(true);
	disconnect.setEnabled(false);
	send.setEnabled(false);
    }
    

    // Executed by its own thread
    void doSend() {
	try {
	    
	    if (sck!=null && !sck.isClosed()) {
		send.setEnabled(false);
		
		PrintWriter out = new PrintWriter(sck.getOutputStream(), true);
		out.print((cmdText.getText() + "\n"));
		out.flush();
		System.out.println("Just sent " + cmdText.getText());
		
		responseText.setText("");
		responseText.setForeground(Color.blue);
		
		BufferedReader in = new BufferedReader(new
		    InputStreamReader(sck.getInputStream()));
 
                System.out.println ("Line :" + in.readLine());
                
		StringBuffer reply = new StringBuffer("");
		
		// HTTP-like response assumed:  
		// First read response and header lines, then body

		// Read header lines with timeout of 10 sec.
		boolean broken = false;  
		sck.setSoTimeout(10000);
		try {
		    while(true) {
			String line = in.readLine();
			if (line==null) {
			    // EOF: Connection closed
			    // This is considered an error in the headers
			    error("Connection broken");
			    broken = true;
			    break;
			}
			reply.append(line + "\n");
			responseText.setText(reply.toString());
			// Here we could pick up Content-Length
			if (line.length()==0)
			    // End of headers -- goto body reception
			    break;
		    }
		} catch (InterruptedIOException e) {
		    error("Timeout occurred during header reception");
		    broken = true;
		} catch (SocketException e) {
		    error("Socket closed during reception");
		    broken = true;
		}

		if (!broken) {
		    // Read body with timeout of 2 sec.
		    sck.setSoTimeout(2000);
		    try {
			while(true) {
			    // Should really be read byte-by-byte
			    String line = in.readLine();
			    if (line==null) {
				// EOF: Connection closed
				// This is considered end-of-body
				broken = true;
				break;
			    }
			    reply.append(line + "\n");
			    responseText.setText(reply.toString());
			}
		    } catch (InterruptedIOException e) {
			// Timeout: We just assume server has finished body
		    } catch (SocketException e) {
			error("Socket closed during reception");
			broken = true;
		    }
		}

		if (broken) {
		    sck.close();
		    connect.setEnabled(true);
		    disconnect.setEnabled(false);
		} else {
		    send.setEnabled(true);
		}
	    }

	} catch (Exception e) {
	    e.printStackTrace();
	}   
    }
    
    class Sender extends Thread {

	public void run() {
	    doSend();
	}
    }

    class ActionAdapter implements ActionListener {
 
       public void actionPerformed(ActionEvent e) {
	   try {
	       // Request a connection initiation
	       if (e.getActionCommand().equals("send")) {
		   // The send operation is done by a thread
		   // in order to enable update of the window
		   // during reception of response
		   if (sender==null || !sender.isAlive()) {
		       sender = new Sender();
		       sender.start();
		   }
	       }
	       
	       if (e.getActionCommand().equals("connect")) {
		   doConnect();
	       }
	       
	       if (e.getActionCommand().equals("disconnect")) {
		   doDisconnect();
	       }
	       
	   } catch (Exception ex) {
	       ex.printStackTrace();
	   }
       }

    }
}




